<?php
/**
 * @file
 * Drush command integration for Lagotto services.
 */

/**
 * Implements hook_drush_command().
 */
function lagotto_services_drush_command() {
  $items['lagotto-lookup-id'] = array(
    'description' => dt('Lookup an article ID on a Lagotto server.'),
    'required-arguments' => 1,
    'aliases' => array('lgid'),
    'arguments' => array(
      'id' => 'The ID being looked up. More than one ID of the same type can be supplied.',
    ),
    'options' => array(
      'type' => 'The type of ID supplied. One of doi, pmc, pmid',
      'server' => 'The URL of the Lagotto server. Overrides the Drupal config.',
      'key' => 'The API Key for the Lagotto server. Overrides the Drupal config.',
    ),
    'examples' => array(
      'drush lagotto-lookup-id 10.7554/eLife.00741' => 'Check the currently defined Lagotto server for details of eLife article e00741',
      'drush lagotto-lookup-id --type=pmc PMC4066238' => 'Check the currently defined Lagotto server for details of PMC ',
      'drush lgid --server=http://alm.example.org/lagotto PMC4066238' => 'Check the currently defined Lagotto server for details of PMC ',
    ),
  );
  return $items;
}

/**
 * Drush callback to lookup a work ID.
 */
function drush_lagotto_services_lagotto_lookup_id() {

  $options = array();

  // Support drush-setup of server.
  $server = drush_get_option('server', '');
  if (preg_match('/^https?:\/\//i', $server)) {
    $server_url = $server . '/api/v5/articles';
    $options['service_url'] = $server_url;
    drupal_set_message($server_url);
  }

  $key = drush_get_option('key', '');
  if (!empty($key)) {
    $options['service_key'] = $key;
  }

  // Get the ID type, validate and convert to Lagotto names.
  switch (drush_get_option('type', 'doi')) {
    default:
    case 'doi':
      $options['id_type'] = 'doi';
      break;

    case 'pmc':
    case 'pmcid':
      $options['id_type'] = 'pmcid';
      break;

    case 'pm':
    case 'pmid':
      $options['id_type'] = 'pmid';
      break;
  }

  // The API can take no more than 50 at a time.
  $ids = array();
  $args = func_get_args();
  foreach ($args as $arg) {
    $ids[] = $arg;
    if (count($ids) >= 48) {
      lagotto_services_do_lookup($ids, $options);
      $ids = array();
    }
  }
  // Stragglers...
  if (count($ids) > 0) {
    lagotto_services_do_lookup($ids, $options);
  }
}

/**
 * Submit an API fetch to the server for the IDs provided.
 *
 * Results are printed on the console using drush_print()
 *
 * @param array $ids
 *   Array of ID values to request. All will be interpreted according
 *   to the 'id_type' key in $options, defaulting to 'doi'.
 * @param array $options
 *   Options defining the fetch. See lagotto_services_fetch_work() for
 *   possible values.
 */
function lagotto_services_do_lookup(array $ids, array $options) {
  $default_options = array(
    'include_detail' => FALSE,
    'quiet' => TRUE,
  );
  $config = array_merge($default_options, $options);

  $result = lagotto_services_fetch_work($ids, $config);
  if ($result == NULL) {
    drush_print(t('Permission denied.'));
    return;
  }

  $records = lagotto_services_records($result);
  if (!empty($records['result'])) {
    drush_print('doi,pmid,pmcid,title');
    foreach ($records['result'] as $record) {
      $fields = array(
        '@doi' => $record['doi'],
        '@pmid' => $record['pmid'],
        '@pmcid' => $record['pmcid'],
        '@ttl' => $record['title'],
      );
      drush_print(lagotto_services_output_tocsv($fields));
    }
  }

  if (!empty($records['error'])) {
    if ($records['error'] == 3000) {
      drush_print(t('No record found.'));
    }
    else {
      drush_print(t('Server returned error : @code : @msg',
        array(
          '@code' => $records['error'],
          '@msg' => $records['message'],
        ))
      );
    }
  }
}

/**
 * Format a fields array as a line of CSV text.
 *
 * @param array $fields
 *   An array of values to print. Key names are not important.
 * @param string $delimiter
 *   The character that separates fields on a line.
 * @param string $wrap
 *   The character that wraps each field's value (e.g. ") if the
 *   field contains whitespace or the delimiter character. If this
 *   character also appears within a field's value, it is doubled.
 * @param bool $null_ok
 *   If True, and a field's value is php-NULL, then output
 *   NULL as the field's value; otherwise output nothing.
 *
 * @return string
 *   A line of the CSV output corresponding to the fields supplied.
 */
function lagotto_services_output_tocsv(array $fields, $delimiter = ',', $wrap = '"', $null_ok = FALSE) {
  $delimiter_esc = preg_quote($delimiter, '/');
  $enclosure_esc = preg_quote($wrap, '/');

  $output = array();
  foreach ($fields as $field) {
    if ($field === NULL && $null_ok) {
      $output[] = 'NULL';
      continue;
    }

    if (preg_match("/(?:${delimiter_esc}|${enclosure_esc}|\\s)/", $field)) {
      $output[] = ($wrap . str_replace($wrap, $wrap . $wrap, $field) . $wrap);
    }
    else {
      $output[] = $field;
    }
  }

  return implode($delimiter, $output);
}
